/*
*
* check_tcp_ping.c
*
* The idea is to use a check on an arbitrary port (default 22)
* and for it to be OKAY if it connects or rejects
* but to be DOWN/CRITICAL if it times-out
*
*
* ======= BASED ON =======
*
* check_ssh.c
*
* Made by (Remi PAULMIER)
* Login   <remi@sinfomic.fr>
*
* Started on  Fri Jul  9 09:18:23 1999 Remi PAULMIER
* Update Thu Jul 22 12:50:04 1999 remi paulmier
* $Id: $
*
*/

#include "config.h"
#include "common.h"
#include "netutils.h"
#include "utils.h"

#define PROGNAME "check_tcp_ping"
#define REVISION "$Revision: 1.1 $"


#define DFL_PORT    22

short port = -1;
char *server_name = NULL;
int verbose = FALSE;
int warn = FALSE;

int process_arguments (int, char **);
int validate_arguments (void);
void print_help (void);
void print_usage (void);

char *name_resolve (char *hostname);
int tcp_connect (char *haddr, short hport);

int
main (int argc, char **argv)
{

	if (process_arguments (argc, argv) == ERROR)
		usage ("Could not parse arguments\n");

	/* initialize alarm signal handling */
	signal (SIGALRM, socket_timeout_alarm_handler);
	alarm (socket_timeout);

	/* tcp_connect exits if error is found */
	tcp_connect (name_resolve (server_name), port);

	alarm (0);

	return (STATE_OK);
}


/* process command-line arguments */
int
process_arguments (int argc, char **argv)
{
	int c;

#ifdef HAVE_GETOPT_H
	int option_index = 0;
	static struct option long_options[] = {
		{"version", no_argument, 0, 'V'},
		{"help", no_argument, 0, 'h'},
		{"verbose", no_argument, 0, 'v'},
		{"timeout", required_argument, 0, 't'},
		{"host", required_argument, 0, 'H'},
		{"warning", no_argument, 0, 'w'},
		{0, 0, 0, 0}
	};
#endif

	if (argc < 2)
		return ERROR;

	for (c = 1; c < argc; c++)
		if (strcmp ("-to", argv[c]) == 0)
			strcpy (argv[c], "-t");

	while (1) {
#ifdef HAVE_GETOPT_H
		c = getopt_long (argc, argv, "+Vhvwt:H:p:", long_options, &option_index);
#else
		c = getopt (argc, argv, "+Vhvwt:H:p:");
#endif
		if (c == -1 || c == EOF)
			break;

		switch (c) {
		case '?':									/* help */
			usage ("");
		case 'V':									/* version */
			print_revision (PROGNAME, REVISION);
			exit (STATE_OK);
		case 'h':									/* help */
			print_help ();
			exit (STATE_OK);
		case 'v':									/* verbose */
			verbose = TRUE;
			break;
		case 'w':									/* warning */
			warn = TRUE;
			break;
		case 't':									/* timeout period */
			if (!is_integer (optarg))
				usage ("Timeout Interval must be an integer!\n\n");
			socket_timeout = atoi (optarg);
			break;
		case 'H':									/* host */
			server_name = optarg;
			break;
		case 'p':									/* port */
			if (is_intpos (optarg)) {
				port = atoi (optarg);
			}
			else {
				printf ("Port number nust be a positive integer: %s\n", optarg);
				usage ("");
			}
		}

	}

	c = optind;
	if (server_name == NULL && argv[c]) {
		server_name = argv[c++];
	}
	else if (port == -1 && argv[c]) {
		if (is_intpos (argv[c])) {
			port = atoi (argv[c++]);
		}
		else {
			print_usage ();
			exit (STATE_UNKNOWN);
		}
	}

	return validate_arguments ();
}

int
validate_arguments (void)
{
	if (server_name == NULL)
		return ERROR;
	if (port == -1)			/* funky, but allows -p to override stray integer in args */
		port = DFL_PORT;
	return OK;
}


/************************************************************************
*
* Resolve hostname into IP address
*
*-----------------------------------------------------------------------*/

char *
name_resolve (char *hostname)
{
	struct hostent *host;

	host = gethostbyname (hostname);
	if (!host) {
		herror (hostname);
		exit (STATE_CRITICAL);
	}
	return (host->h_addr);
}


/************************************************************************
*
* Try to connect to specified machine at specified port
*
*-----------------------------------------------------------------------*/

int
tcp_connect (char *haddr, short hport)
{
	int s;
	struct sockaddr_in addr;
	int addrlen;
	int len;
	int refuse = FALSE;


	addrlen = sizeof (addr);
	memset (&addr, 0, addrlen);
	addr.sin_port = htons (hport);
	addr.sin_family = AF_INET;
	bcopy (haddr, (void *) &addr.sin_addr.s_addr, 4);

	s = socket (AF_INET, SOCK_STREAM, 0);
	if (!s) {
		printf ("socket(): %s for %s:%d\n", strerror (errno), server_name, hport);
		exit (STATE_CRITICAL);
	}

	if (connect (s, (struct sockaddr *) &addr, addrlen)) {
		if (errno != ECONNREFUSED) {
			printf ("connect() errno=%d: %s for %s:%d\n", 
							errno, strerror (errno), server_name, hport);
			exit (STATE_CRITICAL);
		}
		else {
			refuse = TRUE;
		}
	}

	printf ("TCP_PING %s - %s (port %d)%s\n", 
					(refuse && warn)?"warn":"ok",
					server_name, hport,
					(refuse)?" (Connection refused)":"");

	close (s);

	if (warn && refuse)
		exit (STATE_WARNING);
	exit (STATE_OK);

}

void
print_help (void)
{
	print_revision (PROGNAME, REVISION);
	printf ("Copyright (c) 2002 Rodger Allen (r.allen@awayweb.com)\n");
	print_usage ();
	printf ("by default, port is %d\n", DFL_PORT);
	printf ("by default, timeout is %d\n", DEFAULT_SOCKET_TIMEOUT);
}

void
print_usage (void)
{
	printf
		("Usage:\n"
		 " %s [-w] [-t timeout] [-p port] <host>\n"
		 " %s -V prints version info\n"
		 " %s -h prints more detailed help\n"
		 "Options:\n"
		 " -w  issue warning if connection refused (default is OK)\n",
		 PROGNAME, PROGNAME, PROGNAME);
}

/* end of check_tcp_ping.c */
